package com.menagerie.ophelia.database.polycule

import android.content.Context
import androidx.compose.ui.tooling.preview.datasource.LoremIpsum
import androidx.room.Database
import androidx.room.Room
import androidx.room.RoomDatabase
import androidx.sqlite.db.SupportSQLiteDatabase
import com.menagerie.ophelia.database.polycule.entity.Bio
import com.menagerie.ophelia.database.polycule.entity.BioDao
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.launch


@Database(
    version = 2,
    exportSchema = false,
    entities = [
        Bio::class
    ],
)
abstract class PolyculeDatabase : RoomDatabase() {

    abstract fun bioDao(): BioDao

    companion object {
        @Volatile
        private var INSTANCE: PolyculeDatabase? = null

        fun getDatabase(
            context: Context,
            scope: CoroutineScope,
        ): PolyculeDatabase {
            return INSTANCE ?: synchronized(this)
            {
                val instance = Room.databaseBuilder(
                    context.applicationContext,
                    PolyculeDatabase::class.java,
                    "polycule.db"
                )
                    .fallbackToDestructiveMigration()
                    .addCallback(PolyculeDatabaseCallback(scope))
                    .build()
                INSTANCE = instance
                instance
            }
        }
    }

    private class PolyculeDatabaseCallback(
        private val scope: CoroutineScope
    ) : Callback() {

        override fun onCreate(db: SupportSQLiteDatabase) {
            super.onCreate(db)
            INSTANCE?.let { database ->
                scope.launch {
                    populateDatabase(database.bioDao())
                }
            }
        }

        suspend fun populateDatabase(bioDao: BioDao) {
            var desc = LoremIpsum(15).values.toList()
            var init = ""
            repeat(desc.size) {
                init += ("${desc[it]} ")
            }

            var bio = Bio(
                name = "Azea",
                description = init,
            )
            bioDao.insert(bio)

            desc = LoremIpsum(17).values.toList()
            init = ""
            repeat(desc.size) {
                init += ("${desc[it]} ")
            }

            bio = Bio(
                name = "Darkwood Mill",
                description = init,
            )
            bioDao.insert(bio)

            desc = LoremIpsum(42).values.toList()
            init = ""
            repeat(desc.size) {
                init += ("${desc[it]} ")
            }
            bio = Bio(
                name = "Blizzard",
                description = init,
            )
            bioDao.insert(bio)
        }
    }
}