package com.menagerie.ophelia.view

import androidx.compose.runtime.Composable
import androidx.navigation.NavHostController
import androidx.navigation.NavType
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import androidx.navigation.navArgument
import com.menagerie.ophelia.view.biographies.AddBiography
import com.menagerie.ophelia.view.biographies.BioDetailsScreen

@Composable
fun PolyculeApp() {
    val navController = rememberNavController()
    PolyculeNavHost(
        navController = navController
    )
}

@Composable
fun PolyculeNavHost(
    navController: NavHostController
) {
    NavHost(
        navController = navController,
        startDestination = "home"
    ) {
        composable("home") {
            HomeScreen(
                onBioClick = {
                    navController.navigate("bioDetail/${it.id}")
                },
                onAddClick = {
                    navController.navigate("addBio")
                }
            )
        }
        composable(
            "bioDetail/{bioId}",
            arguments = listOf(navArgument("bioId") {
                type = NavType.IntType
            })
        ) {
            BioDetailsScreen(
                id = it.arguments?.getInt("bioId") ?: 0,
                onBackClick = { navController.navigateUp() },
            )
        }
        composable("addBio") {
            AddBiography()
        }
    }
}