package com.menagerie.ophelia.view.biographies

import android.annotation.SuppressLint
import android.widget.Toast
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.text.KeyboardActions
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.Scaffold
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.livedata.observeAsState
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.platform.LocalFocusManager
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.tooling.preview.datasource.LoremIpsum
import androidx.compose.ui.unit.dp
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewmodel.compose.viewModel
import com.menagerie.ophelia.R
import com.menagerie.ophelia.database.polycule.entity.Bio
import com.menagerie.ophelia.database.polycule.entity.viewmodel.BioListViewModel
import com.menagerie.ophelia.view.components.InputFieldComponent
import com.menagerie.ophelia.view.components.fab.FABComponent
import kotlin.random.Random

fun insertBioInDB(bio: Bio?, mBioListViewModel: BioListViewModel) {
    bio?.let {
        mBioListViewModel.upsert(it)
    }
}

@SuppressLint("UnusedMaterial3ScaffoldPaddingParameter")
@Composable
fun AddBiography() {
    val inputViewModel = InputViewModel()
    val context = LocalContext.current
    val mBioListViewModel: BioListViewModel = viewModel(
        factory = BioListViewModel.BioListViewModelFactory()
    )

    Scaffold(
        floatingActionButton = {
            FABComponent(text = "${stringResource(id = R.string.add_bio)}", onClick = {
                insertBioInDB(inputViewModel.bio.value, mBioListViewModel)

                Toast.makeText(context, "Added Bio", Toast.LENGTH_SHORT).show()
            })
        }
    )
    {
        InputFieldState(inputViewModel)
    }
}

@Composable
fun InputFieldState(
    inputViewModel: InputViewModel,
) {
    val bio: Bio by inputViewModel.bio.observeAsState(Bio())
    Column(
        modifier = Modifier.padding(16.dp)
    ) {
        InputField(bio.name) { inputViewModel.onInputChange(it) }
        Spacer(modifier = Modifier.padding(10.dp))
    }
}

@Composable
fun InputField(
    name: String,
    onValChange: ((String) -> Unit)?
) {
    val focusManager = LocalFocusManager.current

    if (onValChange != null) {
        InputFieldComponent(
            text = name,
            onChange = onValChange,
            label = "Enter Bio",
            modifier = Modifier
                .padding(all = 16.dp)
                .fillMaxWidth(),
            keyboardActions = KeyboardActions(onDone = { focusManager.clearFocus() })
        )
    }
}

class InputViewModel : ViewModel() {
    private val _bio: MutableLiveData<Bio> = MutableLiveData(Bio())
    val bio: LiveData<Bio> = _bio
    fun onInputChange(name: String) {
        var desc = LoremIpsum(Random.nextInt(5,100)).values.toList()
        var init = ""
        repeat(desc.size) {
            init += ("${desc[it]} ")
        }
        val newBio = Bio(
            name = name,
            description = init,
        )
        _bio.value = newBio
    }
}
